@@ -4,7 +4,7 @@ from mprop import mproperty
 from subprocess import Popen, PIPE
 import sys
 
-__version__ = "0.99.0"
+__version__ = "0.101.0"
 
 __github_repo = "jupyter-naas/drivers"
 
@@ -352,6 +352,13 @@ def naascredits():
     return NaasCredits()
 
 
+@load_driver()
+def naasevents():
+    from naas_drivers.tools.naas_events import NaasEvents
+
+    return NaasEvents()
+
+
 @load_driver
 def budgetinsight():
     from naas_drivers.tools.budgetinsight import BudgetInsight
